==> ToolChain_STM32_1.0.29.tar.bz: ToolChain_STM32/Examples/example_threading_semaphores.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_example_leds
    leds_initPorts();
#endif
#ifdef EXTENSION_lcd_320x240_olimex
    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,255,200);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
#endif
}
void threading_start() {
    threading_init();

    static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // Mutex being used to wait until task has copied its ThreadInfo
    ThreadInfo.Lock_Init = xSemaphoreCreateMutex();
    Assert(NULL != ThreadInfo.Lock_Init, ec_Malloc);

    // create counting semaphore Producer -> Consumer
    ThreadInfo.SemaphoreProducer2Consumer = xQueueCreateCountingSemaphore(30, 0);
    Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = xQueueCreate(10, sizeof(et_PrintJob_t));
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);
    
    xSemaphoreTake(ThreadInfo.Lock_Init, 0);  // lock structure
    Assert(0 != xTaskCreate( taskPrint,                             //{ function to start as thread
                             (const signed char*) "Print",          // thread name (just for debugging)
                             (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                             (void *) &ThreadInfo,                  // passed as argument to taskValue()
                             (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                             (xTaskHandle*) NULL                    // can return a handle to created task
                             ),
           ec_Malloc); //}
    while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure

    ThreadInfo.DisplayRow = 5;
    for (int AmountProducers = 0; AmountProducers < 2; AmountProducers++) {
        Assert(0 != xTaskCreate( taskProducer,                          //{ function to start as thread
                                 (const signed char*) "Producer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                               ),
               ec_Malloc); //}

        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }

    for (int AmountConsumers= 0; AmountConsumers < 6; AmountConsumers++) {
        Assert(0 != xTaskCreate( taskConsumer,                          //{ function to start as thread
                                 (const signed char*) "Consumer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                                 ),
               ec_Malloc); //}
        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }
    xSemaphoreGive(ThreadInfo.Lock_Init);

    queuePrintJob(ThreadInfo.QueuePrint, 4, 0, "Example");
    queuePrintJob(ThreadInfo.QueuePrint, 3, 1, "Semaphores");
    queuePrintJob(ThreadInfo.QueuePrint, 0, 3, "Q=");
}
void displayQueue(xSemaphoreHandle Semaphore, xQueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    unsigned portBASE_TYPE QueueSize = uxQueueMessagesWaiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '.';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "P>");

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        queuePrintJobV(ThreadInfo.QueuePrint, 6, ThreadInfo.DisplayRow, "+%i ", Value);
        for (int I = 0; I < Value; I++) {
            while (xSemaphoreGive(ThreadInfo.SemaphoreProducer2Consumer) != pdTRUE);
        }

        displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        mSleep(1200);

        Value++;
        if (Value == 10) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

#ifdef EXTENSION_example_leds
    ConfigLED_t ConfigLED = { LED_STAT2, 100, 0 };
    Assert(NULL != xTaskCreate(taskControlLed,                         // function to start as thread
                               (const signed char*) "taskControlLed2", // thread name (just for debugging)
                               (size_t) 2*configMINIMAL_STACK_SIZE,    // stack size
                               (void *) &ConfigLED,                    // passed as argument to taskControlLed()
                               (unsigned portBASE_TYPE) 1,             // task priority (higher values mean more process time)
                               (xTaskHandle*) NULL                     // can return a handle to created task
                               ),
           ec_Malloc);
#endif
    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "C>");

    u32_t Counter = 0;
    while (1) {
        if ( xSemaphoreTake(ThreadInfo.SemaphoreProducer2Consumer, (portTickType) -1) ) {
            Counter++;
#ifdef EXTENSION_example_leds
            ConfigLED.OnPeriod  = 100;
            ConfigLED.OffPeriod = 0;
            mSleep(400);
            ConfigLED.OnPeriod  = 0;
            ConfigLED.OffPeriod = 100;
            mSleep(400);
#else
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "|");
            mSleep(400);
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "-");
            mSleep(400);
            queuePrintJobV(ThreadInfo.QueuePrint, 7, ThreadInfo.DisplayRow, "%d", Counter);
#endif
            displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        }
    }
}
void queuePrintJobV(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (xQueueSend(PrintQueue, &PrintJob, (portTickType) -1) != pdTRUE);
}
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}

void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    while (1) {
        et_PrintJob_t PrintJob;
        if ( xQueueReceive(ThreadInfo.QueuePrint, &PrintJob,  (portTickType) -1) ) {
#ifdef EXTENSION_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions

==> ToolChain_STM32_1.0.31.tar.bz: ToolChain_STM32/Examples/example_threading_semaphores.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_semaphores.h"

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_600_example_leds
    leds_initPorts();
    initPort(PB_LED1, GPIO_Mode_Out_PP);
    initPort(PB_LED2, GPIO_Mode_Out_PP);
#endif
#ifdef EXTENSION_400_lcd_320x240_olimex
    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,255,200);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
#endif
}
void threading_start() {
    threading_init();

    static ets_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // Mutex being used to wait until task has copied its ThreadInfo
    ThreadInfo.Lock_Init = xSemaphoreCreateMutex();
    Assert(NULL != ThreadInfo.Lock_Init, ec_Malloc);

    // create counting semaphore Producer -> Consumer
    ThreadInfo.SemaphoreProducer2Consumer = xQueueCreateCountingSemaphore(30, 0);
    Assert(NULL != ThreadInfo.SemaphoreProducer2Consumer, ec_Malloc);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = xQueueCreate(5, sizeof(et_PrintJob_t));
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);
    
    xSemaphoreTake(ThreadInfo.Lock_Init, 0);  // lock structure
    Assert(0 != xTaskCreate( taskPrint,                             //{ function to start as thread
                             (const signed char*) "Print",          // thread name (just for debugging)
                             (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                             (void *) &ThreadInfo,                  // passed as argument to taskValue()
                             (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                             (xTaskHandle*) NULL                    // can return a handle to created task
                           ),
           ec_Malloc); //}
    while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure

    ThreadInfo.DisplayRow = 5;
#ifdef EXTENSION_600_example_leds
#define AmountProducers 1
#define AmountConsumers 2
#else
#define AmountProducers 1
#define AmountConsumers 6
#endif

    for (int Amount = 0; Amount < AmountProducers; Amount++) {
        Assert(0 != xTaskCreate( taskProducer,                          //{ function to start as thread
                                 (const signed char*) "Producer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                               ),
               ec_Malloc); //}

        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }

    for (int Amount = 0; Amount < AmountConsumers; Amount++) {
#ifdef EXTENSION_600_example_leds
#ifndef PB_LED1
#error PB_LED1 must be defined!
#endif
#ifndef PB_LED2
#error PB_LED2 must be defined!
#endif
        ConfigLED_t ConfigLED = ThreadInfo.ConfigLED;
        switch (Amount) {
        case  0: setLedGPIO(&ThreadInfo.ConfigLED, PB_LED1); break;
        case  1: setLedGPIO(&ThreadInfo.ConfigLED, PB_LED2); break;
        default: setLedGPIO(&ThreadInfo.ConfigLED, NULL, 0); break;
        }
#endif

        Assert(0 != xTaskCreate( taskConsumer,                          //{ function to start as thread
                                 (const signed char*) "Consumer",       // thread name (just for debugging)
                                 (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                 (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                 (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                 (xTaskHandle*) NULL                    // can return a handle to created task
                                 ),
               ec_Malloc); //}
        while (! xSemaphoreTake(ThreadInfo.Lock_Init, (portTickType) -1) );  // wait until task has released structure
        ThreadInfo.DisplayRow++;
    }
    xSemaphoreGive(ThreadInfo.Lock_Init);

    queuePrintJob(ThreadInfo.QueuePrint, 4, 0, "Example");
    queuePrintJob(ThreadInfo.QueuePrint, 3, 1, "Semaphores");
    queuePrintJob(ThreadInfo.QueuePrint, 0, 3, "Q=");

    while (1) mSleep(1000);
}
void displayQueue(xSemaphoreHandle Semaphore, xQueueHandle QueuePrint) {
    char QueueText[11];
    char* Writer = QueueText;
    unsigned portBASE_TYPE QueueSize = uxQueueMessagesWaiting(Semaphore);
    u8_t I = 0;
    for (; (I < QueueSize/3) && (I < 10); I++)
        *Writer++ = '.';
    for (; I < 10; I++)
        *Writer++ = ' ';
    *Writer++ = 0;
    queuePrintJob(QueuePrint, 2, 3, QueueText);
}
void taskProducer(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "P>");

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        queuePrintJobV(ThreadInfo.QueuePrint, 6, ThreadInfo.DisplayRow, "+%i ", Value/10 + 1);
        for (int I = 0; I < Value; I+=10) {
            while (xSemaphoreGive(ThreadInfo.SemaphoreProducer2Consumer) != pdTRUE);
        }

        displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
        mSleep(1200);

        Value++;
        if (Value == 60) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    Assert(NULL != pvParameters,                             ec_NULL);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ((ets_ThreadInfo_t*) pvParameters)->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    queuePrintJob(ThreadInfo.QueuePrint, 2, ThreadInfo.DisplayRow, "C>");

    u32_t Counter = 0;
    while (1) {
        if ( xSemaphoreTake(ThreadInfo.SemaphoreProducer2Consumer, (portTickType) -1) ) {
            Counter++;
#ifdef EXTENSION_600_example_leds
            if (ThreadInfo.ConfigLED.GPIOx != NULL) {
                portSet(ThreadInfo.ConfigLED.GPIOx, ThreadInfo.ConfigLED.Pin);
                mSleep(400);
                portClr(ThreadInfo.ConfigLED.GPIOx, ThreadInfo.ConfigLED.Pin);
                mSleep(400);
            }
#endif
#ifdef EXTENSION_400_lcd_320x240_olimex
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "|");
            mSleep(400);
            queuePrintJob(ThreadInfo.QueuePrint, 5, ThreadInfo.DisplayRow, "-");
            mSleep(400);
            queuePrintJobV(ThreadInfo.QueuePrint, 7, ThreadInfo.DisplayRow, "%d", Counter);
            displayQueue(ThreadInfo.SemaphoreProducer2Consumer, ThreadInfo.QueuePrint);
#endif
        }
    }

    while (1) mSleep(1000);
}
void queuePrintJobV(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (xQueueSend(PrintQueue, &PrintJob, (portTickType) -1) != pdTRUE);
}
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text) {
    queuePrintJobV(PrintQueue, Column, Row, Text, 0);
}
void taskPrint(void *pvParameters) {
    ets_ThreadInfo_t* ThreadInfoPtr = (ets_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfoPtr, ec_NULL);
    Assert(NULL != ThreadInfoPtr->SemaphoreProducer2Consumer, ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->QueuePrint,                 ec_InvalidArgument);
    Assert(NULL != ThreadInfoPtr->Lock_Init,                  ec_InvalidArgument);

    ets_ThreadInfo_t ThreadInfo;
    memcpy(&ThreadInfo, pvParameters, sizeof(ets_ThreadInfo_t));
    xSemaphoreGive(ThreadInfo.Lock_Init); // task creator may now reuse *pvParameters

    while (1) {
        et_PrintJob_t PrintJob;
        if ( xQueueReceive(ThreadInfo.QueuePrint, &PrintJob,  (portTickType) -1) ) {
#ifdef EXTENSION_400_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions
